# -*- coding: utf-8 -*-
import fedmsg.consumers

from . import models, connect_db
from . import lib

PAGURE_MAIN_REPO = '{base}{name}.git'
PAGURE_FORK_REPO = '{base}forks/{user}/{name}.git'


class Integrator(fedmsg.consumers.FedmsgConsumer):
    topic = [
        'io.pagure.prod.pagure.pull-request.comment.added',
        'io.pagure.prod.pagure.pull-request.new',
        'org.fedoraproject.prod.jenkins.build',
    ]

    config_key = 'integrator.enabled'

    def __init__(self, hub):
        super(Integrator, self).__init__(hub)
        self.session = connect_db()

    def consume(self, msg):
        topic, msg = msg['topic'], msg['body']
        self.log.info("Received %r, %r", topic, msg.get('msg_id', None))
        msg = msg['msg']
        try:
            if topic.endswith('.pull-request.comment.added'):
                if is_rebase(msg):
                    self.trigger_build(msg)
            elif topic.endswith('.pull-request.new'):
                self.trigger_build(msg)
            else:
                self.process_build(msg)
        except models.ConfigNotFound as exc:
            self.log.info('Unconfigured project %r', str(exc))

    def trigger_build(self, msg):
        pr_id = msg['pullrequest']['id']
        project = msg['pullrequest']['project']['name']
        branch = msg['pullrequest']['branch_from']
        branch_to = msg['pullrequest']['branch']

        for cfg in models.get_configs(project, models.Service.PAGURE):
            repo = msg['pullrequest'].get('remote_git') or get_repo(cfg, msg)

            self.log.info("Trigger on %s PR #%s from %s: %s",
                          project, pr_id, repo, branch)

            lib.process_pr(self.session, self.log, cfg, pr_id, repo, branch, branch_to)

    def process_build(self, msg):
        for cfg in models.get_configs(msg['project'], models.Service.JENKINS):
            lib.process_build(self.session, self.log, cfg, msg['build'])


def get_repo(cfg, msg):
    url = PAGURE_MAIN_REPO
    if msg['pullrequest']['repo_from']['parent']:
        url = PAGURE_FORK_REPO
    return url.format(
        base=cfg.pagure_url,
        user=msg['pullrequest']['repo_from']['user']['name'],
        name=msg['pullrequest']['repo_from']['name'])


def is_rebase(msg):
    if msg['pullrequest']['status'] != 'Open':
        return False
    try:
        return msg['pullrequest']['comments'][-1]['notification']
    except (IndexError, KeyError):
        return False
