# -*- coding: utf-8 -*-
import requests
import jenkins

from . import models

PAGURE_URL = '{base}api/0/{repo}/pull-request/{pr}/flag'
JENKINS_TRIGGER_URL = '{base}job/{project}/buildWithParameters'


def process_pr(session, logger, cfg, pr_id, repo, branch, branch_to):
    ident = models.create_run(session, cfg.name, pr_id)
    post_data(logger,
              JENKINS_TRIGGER_URL.format(base=cfg.jenkins_url, project=cfg.jenkins_name),
              {'token': cfg.jenkins_token,
               'cause': ident,
               'REPO': repo,
               'BRANCH': branch,
               'BRANCH_TO': branch_to})


def process_build(session, logger, cfg, build_id):
    #  Get details from Jenkins
    jenk = jenkins.Jenkins(cfg.jenkins_url)
    build_info = jenk.get_build_info(cfg.jenkins_name, build_id)
    result = build_info['result']
    url = build_info['url']

    cause = None

    for action in build_info['actions']:
        for c in action.get('causes', []):
            try:
                cause = int(c['note'])
            except (KeyError, ValueError):
                continue

    if not cause:
        logger.info('Not a PR check')
        return

    run, changed = models.mark_run(session, cause, result == 'SUCCESS', build_id)
    if not changed:
        logger.info('Result already saved for %s PR %d: %s',
                    cfg.pagure_name, run.number, result)
        return

    # Comment in Pagure
    logger.info('Updating %s PR %d: %s', cfg.pagure_name, run.number, result)
    try:
        post_flag(logger, cfg.display_name, cfg.pagure_url, cfg.pagure_token,
                  cfg.pagure_name, run.number, result, url)
    except KeyError as exc:
        logger.warning('Unknown build status', exc_info=exc)


def post_flag(logger, name, base, token, repo, pr, result, url):
    comment, percent = {
        'SUCCESS': ('Build successful', 100),
        'FAILURE': ('Build failed', 0),
    }[result]
    payload = {
        'username': name,
        'percent': percent,
        'comment': comment,
        'url': url,
    }
    post_data(logger, PAGURE_URL.format(base=base, repo=repo, pr=pr), payload,
              headers={'Authorization': 'token ' + token})


def post_data(logger, *args, **kwargs):
    resp = requests.post(*args, **kwargs)
    logger.debug('Received response status %s', resp.status_code)
    if resp.status_code < 200 or resp.status_code >= 300:
        logger.error('Network request failed: %d: %s', resp.status_code, resp.text)
